package dev.pfaff.altargui.pathfinder;

import java.lang.foreign.Arena;
import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.pathfinder.Pathfinder.*;
import static java.lang.foreign.ValueLayout.JAVA_FLOAT;

public record PFCanvasRef(MemorySegment ptr) {
	public PFSceneRef takeScene() {
		try {
			var arena = Arena.ofAuto();
			return new PFSceneRef(manageNativeObject(arena,
					(MemorySegment) MH_PFCanvasTakeScene.invokeExact(arena, this.ptr()),
					ptr -> MH_PFSceneDeinit.invokeExact(ptr)));
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @param font the font to set. It is borrowed during the call, but is left usable afterwards.
	 */
	public void setFont(FKFontRef font) {
		try {
			int result = (int) MH_PFCanvasSetFont.invokeExact(this.ptr(), font.ptr());
			if (result != 0) throw new RuntimeException(STR."Failed to set font: \{result}");
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void setFontSize(float size) {
		try {
			MH_PFCanvasSetFontSize.invokeExact(this.ptr(), size);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void fillText(String text, float x, float y) {
		try (var arena = Arena.ofConfined()) {
			var textBytes = arena.allocateFrom(text);
			var position = arena.allocateFrom(JAVA_FLOAT, x, y).asSlice(0, ML_PFVector2F);
			MH_PFCanvasFillText.invokeExact(this.ptr(), textBytes, textBytes.byteSize() - 1, position);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void strokeText(String text, float x, float y) {
		try (var arena = Arena.ofConfined()) {
			var textBytes = arena.allocateFrom(text);
			var position = arena.allocateFrom(JAVA_FLOAT, x, y).asSlice(0, ML_PFVector2F);
			MH_PFCanvasStrokeText.invokeExact(this.ptr(), textBytes, textBytes.byteSize() - 1, position);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @return the width of the text
	 */
	public float measureText(String text) {
		try (var arena = Arena.ofConfined()) {
			var textBytes = arena.allocateFrom(text);
			var metrics = arena.allocate(ML_PFTextMetrics);
			MH_PFCanvasMeasureText.invokeExact(this.ptr(), textBytes, textBytes.byteSize() - 1, metrics);
			return (float) VH_PFTextMetrics_width.get(metrics);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void fillRect(float x1, float y1, float x2, float y2) {
		try (var arena = Arena.ofConfined()) {
			var rect = arena.allocateFrom(JAVA_FLOAT, x1, y1, x2, y2).asSlice(0, ML_PFRectF);
			MH_PFCanvasFillRect.invokeExact(this.ptr(), rect);
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void setFillStyle(PFFillStyleRef style) {
		try {
			MH_PFCanvasSetFillStyle.invokeExact(this.ptr(), style.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void setStrokeStyle(PFFillStyleRef style) {
		try {
			MH_PFCanvasSetStrokeStyle.invokeExact(this.ptr(), style.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void clear() {
		try {
			MH_PFCanvasClear.invokeExact(this.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
