package dev.pfaff.altargui.pathfinder;

import java.lang.foreign.MemorySegment;

import static dev.pfaff.altargui.pathfinder.Pathfinder.*;

public record PFSceneProxyRef(MemorySegment ptr) {
	public static PFSceneProxyRef createWithSequentialExecutor(PFRendererMode mode) {
		return new PFSceneProxyRef(constructNativeObject(ML_PFSceneProxy,
				ptr -> (int) MH_PFSceneProxyInitWithSequentialExecutor.invokeExact(ptr, mode.value),
				ptr -> MH_PFSceneProxyDeinit.invokeExact(ptr)));
	}

	public static PFSceneProxyRef createWithRayonExecutor(PFRendererMode mode) {
		return new PFSceneProxyRef(constructNativeObject(ML_PFSceneProxy,
				ptr -> (int) MH_PFSceneProxyInitWithRayonExecutor.invokeExact(ptr, mode.value),
				ptr -> MH_PFSceneProxyDeinit.invokeExact(ptr)));
	}

	/**
	 * The {@code scene} will be moved out of the reference by this call.
	 */
	public void replaceScene(PFSceneRef scene) {
		try {
			MH_PFSceneProxyReplaceScene.invokeExact(this.ptr, scene.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}

	public void buildAndRenderGL(PFGLRendererRef renderer, PFBuildOptionsRef options) {
		try {
			MH_PFSceneProxyBuildAndRenderGL.invokeExact(this.ptr(), renderer.ptr(), options.ptr());
		} catch (Throwable e) {
			throw new RuntimeException(e);
		}
	}
}
